==> gcl-0.2.32.tgz: package/src/coooders/collection/pool.js <==
import V from '../common/tool';
/**
 * 仅限于池方式调用的方法
 * @author(baibing)
 * @param('size','')
 * @param(func:创建对象的方法)
 * @param(waitTime,过期时间)
 */
export const pool = class {
    constructor(size = 10, func, waitTime = 10) {
        const that = this;
        const { _, __ } = pri(that, {
            size: size,
            func: func,
            waitTime: waitTime,
            data: [],
            have: [],
            wait: [],
            use: {},
            KEY: Symbol('_____poolid'),
            addUse: (v) => {
                const { _, __ } = pri(that);
                if (v) {
                    if (!V.isValid(v[__.KEY])) {
                        v[__.KEY] = V.random();
                    }
                    __.use[v[__.KEY]] = v;
                }
            },
            delUse: (v) => {
                const { _, __ } = pri(that);
                if (v) {
                    if (V.isValid(v[__.KEY]) && V.isValid(__.use[v[__.KEY]])) {
                        delete __.use[v[__.KEY]];
                    }
                }
            }
        });
        __.clear = new clearer(_, __);
        __.clear.start();
    }
    getValue() {
        const { _, __ } = pri(this);
        return V.callback(call => {
            let v = __.have.pop();
            if (v) {
                v = v.value;
                __.addUse(v);
                call(null, v);
            } else if (__.data.length < __.size) {
                V.callback2(__.func, __).then(v => {
                    __.addUse(v);
                    __.data.push(v);
                    call(null, v);
                });
            } else {
                __.wait.push({ endDate: new Date().add('s', __.waitTime), value: call, dispose: call });
            }
        });
    }
    setValue(v) {
        const { _, __ } = pri(this);
        __.delUse(v);
        if (V.isValid(v) && V.isValid(v[__.KEY])) {
            if (__.wait.length > 0) //仅针对getValue异步操作有效
                V.once(() => {
                __.addUse(v);
                __.wait.shift().value(null, v);
            });
            else
                __.have.push({ endDate: new Date().add('s', __.waitTime), value: v, dispose: function() { if (v.dispose) v.dispose(); } });
        }
    }
    dispose() {
        const { _, __ } = pri(this);
        __.clear.stop();
        while (__.have.length > 0) {
            let val = __.have.shift();
            if (val.dispose) {
                V.tryC(val.dispose)
                val = null;
            }
        }
        __.data = __.data.splice(0);
    }
};
export default { pool };
const pri = V.pris();
class clearer {
    constructor(obj, objpri) {
        const { _, __ } = pri(this, { obj: obj, pri: objpri });
        this.tid = null;
    }
    check(datas) {
        let endDate = datas.length > 0 ? datas[0].endDate : null;
        if (V.isValid(endDate) && new Date().diff('ms', endDate) > 0) {
            return false;
        } else {
            while (datas.length > 0) {
                let val = datas.shift();
                if (val.dispose) {
                    V.tryC(val.dispose);
                    val = null;
                }
            }
            return true;
        }
    }
    start() {
        const { _, __ } = pri(this);
        _.tid = global.setTimeout(() => {
                let ret = _.check(__.pri.data);
                if (ret) {
                    [...__.pri.data] = __.pri.use;
                    _.start();
                }
                _.check(__.pri.wait);
            },
            500);
    }
    stop() {
        const _ = this;
        if (_.tid)
            global.clearTimeout(_.tid);
        _.tid = null;
    }
};

==> gcl-0.2.87.tgz: package/src/coooders/collection/pool.js <==
import V from '../common/tool';
/**
 * 仅限于池方式调用的方法
 * @author(baibing)
 * @param('size','')
 * @param(func:创建对象的方法)
 * @param(waitTime,过期时间)
 */
export const Pool = class {
    constructor(size = 10, func, waitTime = 10) {
        const that = this;
        const { _, __ } = pri(that, {
            size: size,
            func: func,
            waitTime: waitTime,
            data: [],
            have: [],
            wait: [],
            use: {},
            KEY: Symbol('_____poolid'),
            addUse: (v) => {
                const { _, __ } = pri(that);
                if (v) {
                    if (!V.isValid(v[__.KEY])) {
                        v[__.KEY] = V.random();
                    }
                    __.use[v[__.KEY]] = v;
                }
            },
            delUse: (v) => {
                const { _, __ } = pri(that);
                if (v) {
                    if (V.isValid(v[__.KEY]) && V.isValid(__.use[v[__.KEY]])) {
                        delete __.use[v[__.KEY]];
                    }
                }
            }
        });
        __.clear = new Clearer(_, __);
        __.clear.start();
    }
    getValue() {
        const { _, __ } = pri(this);
        return V.callback(call => {
            let v = __.have.pop();
            if (v) {
                v = v.value;
                __.addUse(v);
                call(null, v);
            } else if (__.data.length < __.size) {
                V.callback2(__.func, __).then(v => {
                    __.addUse(v);
                    __.data.push(v);
                    call(null, v);
                });
            } else {
                __.wait.push({ endDate: new Date().add('s', __.waitTime), value: call, dispose: call });
            }
        });
    }
    setValue(v) {
        const { _, __ } = pri(this);
        __.delUse(v);
        if (V.isValid(v) && V.isValid(v[__.KEY])) {
            if (__.wait.length > 0) //仅针对getValue异步操作有效
                V.once(() => {
                __.addUse(v);
                __.wait.shift().value(null, v);
            });
            else
                __.have.push({ endDate: new Date().add('s', __.waitTime), value: v, dispose: function() { if (v.dispose) v.dispose(); } });
        }
    }
    dispose() {
        const { _, __ } = pri(this);
        __.clear.stop();
        while (__.have.length > 0) {
            let val = __.have.shift();
            if (val.dispose) {
                V.tryC(val.dispose)
                val = null;
            }
        }
        __.data = __.data.splice(0);
    }
};
export default { Pool };
const pri = V.pris();
class Clearer {
    constructor(obj, objpri) {
        const { _, __ } = pri(this, { obj: obj, pri: objpri });
        this.tid = null;
    }
    check(datas) {
        let endDate = datas.length > 0 ? datas[0].endDate : null;
        if (V.isValid(endDate) && new Date().diff('ms', endDate) > 0) {
            return false;
        } else {
            while (datas.length > 0) {
                let val = datas.shift();
                if (val.dispose) {
                    V.tryC(val.dispose);
                    val = null;
                }
            }
            return true;
        }
    }
    start() {
        const { _, __ } = pri(this);
        _.tid = global.setTimeout(() => {
                let ret = _.check(__.pri.data);
                if (ret) {
                    [...__.pri.data] = __.pri.use;
                    _.start();
                }
                _.check(__.pri.wait);
            },
            500);
    }
    stop() {
        const _ = this;
        if (_.tid)
            global.clearTimeout(_.tid);
        _.tid = null;
    }
};
